function BeneCast:OnInitialize()

	self:FillSpellInfo();
	self:GenerateAllSpellConfigs();
	
	-- Set BC_class to the player's class
	_, wv.PlayerClass = UnitClass('player');
	
	wv.DefaultConfig = self:GetDefaultConfig();
	self.db = LibStub('AceDB-3.0'):New('BeneCastDB',{ char = wv.DefaultConfig });
	LibStub('AceConfig-3.0'):RegisterOptionsTable('BeneCast', options, {'benecast', 'bc'});
			
	-- Reset the data if there BeneCast is severly outdated
	if (not self.db.char.Profiles or table.getn(self.db.char.Profiles) == 0) or
	   (not self.db.char.Setup or table.getn(self.db.char.Setup) == 0) or
	   (not self.db.char.Setup.version or (self.db.char.Setup.version < self.Constants.DBVersion))
	then
		self:Print('Resetting settings');
		self.db.char = _CopyTable(wv.DefaultConfig,true);
	end
	BeneCastSetup = BeneCast.db.char.Setup;
	BeneCastProfiles = BeneCast.db.char.Profiles;
	
	-- Load up spell data (and other data) once AddOn has been loaded
	self:ResetData();
	
	-- Restore minimapbutton
	if MinimapButton then
		if MinimapButton:GetMinimapButtonPosition() == nil then
			MinimapButton:SetMinimapButtonPosition(random(0,360));
		end
		MinimapButton:UpdatePosition();
		if BeneCastSetup.hideminimapbutton then
			MinimapButton:Hide();
		else
			MinimapButton:Show();
		end
	end
	
end

function BeneCast:OnEnable()

	-- Register for Events
	self:RegisterEvent('ADDON_LOADED');
	self:RegisterEvent('PLAYER_ENTERING_WORLD');
	self:RegisterEvent('PLAYER_LEAVING_WORLD');
	self:RegisterEvent('VARIABLES_LOADED');
	self:RegisterEvent('SPELLS_CHANGED');
	self:RegisterEvent('PARTY_MEMBERS_CHANGED');
	self:RegisterEvent('PLAYER_TARGET_CHANGED');
	self:RegisterEvent('PLAYER_FOCUS_CHANGED');
	self:RegisterEvent('UNIT_AURA');
	self:RegisterEvent('PLAYER_AURAS_CHANGED');
	self:RegisterEvent('UNIT_AURASTATE');
	self:RegisterEvent('UPDATE_SHAPESHIFT_FORMS');
	self:RegisterEvent('PLAYER_ALIVE');
	self:RegisterEvent('PLAYER_DEAD');
	self:RegisterEvent('PLAYER_UNGHOST');
	self:RegisterEvent('UNIT_HEALTH');
	self:RegisterEvent('UNIT_PET');
	self:RegisterEvent('UNIT_TARGET');
	self:RegisterEvent('PLAYER_REGEN_DISABLED');
	self:RegisterEvent('PLAYER_REGEN_ENABLED');
		
	-- Restore disabled timers
	for i, timername in pairs(wv.DisabledTimers) do
		BeneCast.Timers[timername]:Enable();
	end
	_ClearTable(wv.DisabledTimers, false);

end

function BeneCast:OnDisable()

	for timername, timer in pairs(BeneCast.Timers) do
		if timer.handle then
			table.add(wv.DisabledTimers, timername);
		end
		timer:Disable();
	end
	self:CancelAllTimers(); -- just to be safe
	self:UnRegisterAllEvents();

end

function BeneCast:GetProfile(pf, unit)
	if not BeneCastProfiles then
		-- Addon hasn't initialized yet
		return;
	end
	
	if not BeneCastProfiles[profileid] then
		self:ResetProfile(profileid);
	end
	
	if not BeneCastProfiles[profileid].snapto then
		BeneCastProfiles[profileid].snapto = {};
	end
	
	if unit then
		return BeneCastProfiles[profileid], BeneCastProfiles[profileid].snapto[unit];
	else
		return BeneCastProfiles[profileid];
	end
end

function BeneCast:GetCurrentProfileID()
	if wv.ConsiderInCombat then
		return 'combat';
	else
		return 'ooc';
	end
end

-- Function to resize the buttons of a profile
function BeneCast:ResizeButtonsForProfile(pf)

	if wv.Panels[pf] == nil then
		wv.Panels[pf] = {};
	end
	
	-- Resize the buttons of every panel
	for unit, panel in pairs (wv.Panels[pf]) do
		panel:ResizeButtons(i);
	end
	
end

-- Resets data tables
function BeneCast:ResetData()

	-- Clear the BC_spell_data table
	_ClearTable(wv.Spells, true);
	
	-- Reload the spell data
	self:FillSpellInfo();
	
	self:GenerateAllSpellConfigs();
end

-- Function to make a shiny new, blank config
--  Doesn't mean that it will be used to actually reset the config but one does need it around
function BeneCast:GetDefaultConfig()

	-- Reset the BeneCastProfiles-table
	-- assume this will be added to the char-part of an AceDB-database
	local newconfig = {
		Setup = {
			debug = nil,
			version = BeneCast.Constants.DBVersion,
			snapto = 'BeneCast',
			hideminimapbutton = false,
			unlockbuttons = false,
			-- Stores the position of the minimap button (needed for the minimap-code I swiped from Bongos2)
			minimappos = 0,
		},
		Profiles = {
			ooc = {
				name = L['Out of combat'],
			},
			combat = {
				name = L['In combat'],
			},
		},
	};
	
	self:ResetConfig(newconfig.Profiles.ooc, 'ooc');
	self:ResetConfig(newconfig.Profiles.combat, 'combat');
	
	return newconfig;
	
end

function BeneCast:ResetConfig(profile, defaultname)

	local profname = profile.name;
	_ClearTable(profile, true);
	if profname == nil then
		profname = defaultname;
	end 
	profile.name = profname;
	
	profile.snapto = {}; -- this will store the panel's positions, k = unit, v = { x = ??, y = ??, ... }
	profile.config = {}; -- this will store the spells configurations, k = groupname, v = { enabled = ?? }
	for i, v in pairs(BeneCast.Constants.Configs) do
		profile.config[v] = {};
	end

	profile.showallbuttons = false;
	profile.flashasfade = false;
	profile.showtooltips = true;
	profile.showtooltipname = false;
	profile.cascadeplayer = true;
	profile.showpets = false;
	profile.checkrange = false; 
	profile.showtargettarget = true;
	profile.buttonsize = 36;
	profile.orientation = 'RIGHT';
	profile.unlockbuttons = false;

end

function BeneCast:Debug(msg)

	if BeneCastSetup.debug then
		self:Print('Debug: ' .. msg);
	end

end

function BeneCast:AddButtonThatFlashes(button)

	if not mv.ButtonsThatFlash[pf] then
		mv.ButtonsThatFlash[pf] = {};
	end
	if not mv.ButtonsThatFlash[pf][unit] then
		mv.ButtonsThatFlash[pf][unit] = {};
	end
	table.add(mv.ButtonsThatFlash[pf][unit],button);
	BeneCast.Timers.FlashButtons:Enable();

end

function BeneCast:UpdateAllPanels()

	for pfid, pfpanels in pairs(wv.Panels) do
		for unit, panel in pairs(pfpanels) do
			panel:Update();
		end
	end

end

function BeneCast:UpdatePanelsForUnit(unit)
	
	if unit then
		local panel;
		for pfid, pfpanels in pairs(wv.Panels) do
			panel = pfpanels[unit];
			if panel then
				panel:Update();
			end
		end
	end
	
end

function BeneCast:UpdatePanelsForProfile(pfid)
	
	if pfid then
		if wv.Panels[pfid] == nil then
			wv.Panels[pfid] = {};
		end
	
		-- Update the buttons of every panel
		for unit, panel in pairs (wv.Panels[pfid]) do
			panel:Update(i);
		end
	end
	
end

function BeneCast:AddUnitToUpdate(unit, when)

	local utu = wv.UnitsToUpdate;
	if utu[unit] ~= nil then
		utu[unit] = when;
	else
		utu[unit] = when;
		utu.count = utu.count + 1;
	end
	if utu.count > 0 then
		BeneCast.Timers.UpdatePanels:Enable();
	end

end