-- BeneCast by Wintrow

-- Props to Skurel's original BeneCast
-- This version is nearly completely rebuilt and rewritten though
-- Makes clickable buttons that casts beneficial spells on party members and the current target

-- This lua-file is both to initialize BeneCast with crucial variables as well as constants used throughout the code

BeneCast = LibStub('AceAddon-3.0'):NewAddon('BeneCast','AceConsole-3.0','AceEvent-3.0','AceTimer-3.0');

BeneCast.Constants = {
	-- This is used to reset the DB if severely outdated
	DBVersion = 10.0,
	-- This is the version of the addon, shown at the config screen
	Version = '10.0.5',
	
	Name = 'BeneCast',
	-- Possible targets to attach a panel to
	Targets = {	'player', 'party1', 'party2', 'party3', 'party4', 'pet', 'partypet1', 'partypet2', 'partypet3', 'partypet4', 'target', 'targettarget', 'focus' },
	-- How spells should be sorted
	TypesSorted = { 'other', 'rez', 'brez', 'heal', 'hot', 'consume', 'buff', 'cure' },
	-- To easily find the pet of a certain unit
	PetsByOwner = {
		player = 'pet',		
	},
	-- To easily find the owner of a certain pet
	OwnersByPet = {
		pet = 'player',
	},
	-- To find the next orientation when doing "change orientation" on a panel
	Orientations = {
		RIGHT = {
			translation = L['Right'],
			next = 'DOWN'
		},
		DOWN = {
			translation = L['Down'],
			next = 'LEFT',
		},
		LEFT = {
			translation = L['Left'],
			next = 'UP',
		},
		UP = {
			translation = L['Up'],
			next = 'RIGHT',
		},
	},
	-- All classes
	Classes = { 'DEATHKNIGHT','DRUID','HUNTER','MAGE','PALADIN','PRIEST','ROGUE','SHAMAN','WARLOCK','WARRIOR' },
	-- Lists the targettypes that a config can list
	TargetTypes = {
		all       = { player = true, pet = true, party = true, partypet = true, other = true },
		allonly   = { pet = true, party = true, partypet = true, other = true },
		self      = { player = true },
		pet       = { pet = true },
		group     = { player = true, party = true, partypet = true },
		grouponly = { party = true, partypet = true },
	},
	-- All configs
	Configs = { 'player', 'pet', 'party', 'partypet', 'other' },
};
for i = 1, 4 do
	BeneCast.Constants.PetsByOwner['party'..i] = 'partypet'..i;
	BeneCast.Constants.OwnersByPet['partypet'..i] = 'party'..i;
end

-- The workvars contains all the tables and variables the code uses that would otherwise be "floating around"
-- These do not need to be saved across sessions as they are built up every time
-- Some of them are initialized to a certain value though
BeneCast.WorkVars = {
	PlayerClass = '',
	AttachedTargets = {}, -- Array to hold the list of targets we are currently attached to
	                      -- A fix by Shimavak on the 8.xx-version that I carried through to this one, prevents overupdating a panel
	ButtonsThatFlash = {}, -- Holds all buttons that are supposed to flash
	UnitsToUpdate = { -- It is preferred that the code schedules the update of a frame to avoid overupdating
		count = 0,
	},
	ConsiderInCombat = false, -- Right before combat an event is fired that allows the addon to change stuff before the lockdown is in effect
	                          -- This var indicates that this is the case
	Spells = {}, -- Holds the currently known spells as parsed from the spellbook
	SpellInfo = {}, -- Holds the translated version of the spells-table in SpellInfo.lua
	TempEnchantsInfo = {}, -- Holds a smaller version of the Spells-table with only the known enchants, for fast access
	Panels = {}, -- Holds all the created panels as wv.Panels[profileid][unit] = panel
	OldButtons = {}, -- When buttons are no longer needed they are kept here for future recycling purposes
	MHWeaponEnchant = nil, -- Holds info on what (known) enchant is on the Mainhand
	OHWeaponEnchant = nil, -- Holds info on what (known) enchant is on the Offhand
	CurrentStance = 'none', -- Holds info on what stance the player is (this limits the available spells)
	DefaultConfig = nil, -- Filled up when addon is loaded
	SnapToFound = {}, -- Holds the addons that are loaded which exist in the snapto-table in SnapTo.lua
	AttachFrames = nil, -- Holds a variable to remember to re-attach all frames at the next convenience
	DisabledTimers = {}, -- Stores timers that were stopped when addon was disabled
};
-- Holds the template for a panel
BeneCast.Panel = {
	unit = nil,
	pf = nil,
	buttons = {},
	DropDown = {},
};
-- Holds the template for a button
BeneCast.Button = {
	NormalTexture = nil,
	Cooldown = nil,
	Fade = nil,
	Range = nil,
	Duration = nil,
	unit = nil,
	pf = nil,
	seqno = nil,
};
BeneCast.Assist = {
	assist = true,
	target = 'allonly',
	name = L['Assist'],
	group = L['Assist'],
	type = 'other',
	icon = '', -- TODO: needs an icon
};
BeneCast.Focus = {
	focus = true,
	target = 'all',
	name = L['Focus'],
	group = L['Focus'],
	type = 'other',
	icon = '', -- TODO: needs an icon
};
-- Holds the database for BeneCastSetup and BeneCastProfiles
BeneCast.db = {};

-- Some handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastSetup = nil;
local BeneCastProfiles = nil;